// Fancy Search by JingotheCat!
// Modified 10/7/21 to add in load/save tag feature plus the ability to select multiple search tags across XMP groups.

var DATA_CONFIG = "imatch.DBSearch.config";
const DATA_CONFIG_VERSION = 1;

// config includes the tags to save - it will be saved into the user DB.
var config = {
  version: DATA_CONFIG_VERSION,
  scope: "db",
  fwMode: "fwm-result",
  tags: [],
};

// We set this flag for the "progress button"
var running = false;

// We use a fairly large page size.
// This speeds up processing a lot for large databases.
var pageSize = 5000;

var currentPage = 0;
var searchScope = "";
var filesTotal = 0;
var filesWindow = 0;
var filesRange = 0;
var resultWindowInstanceId = 0;
var matches = [];
var sourceFiles = [];
var bulkMatchFileNames = [];
var bulkMatchNameOnly = false;
var maintags = [];

// Hide the "progress button" initially
$("#btn-running").hide();

$(document).ready(function () {
  // Get the info about the current user.
  // Then load the stored configuration data.
  IMatch.userInfo().then(function (response) {
    DATA_CONFIG = DATA_CONFIG.replace("{u}", response.userName);
    IMWS.dataGet({ name: DATA_CONFIG })
      .then(function (response) {
        var data = JSON.parse(response.value);
        config = data;
      })
      .always(function () {
        // Whether or not we had stored configuration data,
        // initialize the controls
        $("#digit-match-length").val(config.digitMatch);
        $("#sim-max-distance").val(config.simMatch);
        $('#search-scope option[value="' + config.scope + '"]').prop(
          "selected",
          true
        );
        //console.log("config: " + JSON.stringify(config, null, 2));
      });
  });

  IMatchTranslate.init(IMatchTranslate.getUserLanguage(), "en", "res/");

  // Get the total files in the current scope/database for use later in the results header
  IMatch.get(`info`).then(function (imwsInfo) {
    filesTotal = imwsInfo.database.files;
  });

  IMWS.get("v1/files", {
    idlist: IMatch.idlist.fileWindowFiles,
    fields: "id",
  }).then(function (response) {
    filesWindow = response.files.length;
  });

  setupMonitoring();

  // Bind button handlers from HTML file - each button click is its own function below

  // Load Search Tags from the DB
  $("#btn-ld-results").click(function (e) {
    loadConfig();
    e.preventDefault();
    return false;
  });

  // Remove Selected Tags from the Tag List
  $("#btn-deltag").click(function (e) {
    var selectbox = document.getElementById("srctags");
    var i;
    for (i = selectbox.options.length - 1; i >= 0; i--) {
      if (selectbox.options[i].selected) selectbox.remove(i);
    }
    e.preventDefault();
    return false;
  });

// Clear all Tags from the Tag List
  $("#btn-clear").click(function (e) {
    $("#srctags").empty();
    config.tags = "";
    maintags = [];
    e.preventDefault();
    return false;
  });

// Save the Search Tags into the config for future sessions
  $("#btn-sv-results").click(function (e) {
    saveConfig();
    e.preventDefault();
    return false;
  });

// Perform the search 
  $("#btn-all-match").click(function (e) {
    if (!running) {
      runSearch();
    }
    e.preventDefault();
    return false;
  });

  // Save each single or group of tags to build a search list
  // the code sorts the selected tags and copies them to the search tags list.
  $("#btn-save").click(function (e) {
    var select1 = $("#tags > option:selected").sort().clone();
    $("#srctags").append(select1);
    $("#srctags").val($("#tags").val());
    return false;
  });

  $("#search-scope").on("change", (event) => {
    config.scope = $("#search-scope").val();
    if (config.scope == "db") {
      $("#fw-mode-group").hide();
    } else {
      $("#fw-mode-group").show();
    }
  });

  // Find Tags Button -> compile a list of tags based on the pattern provided - this list can be used for narrowing down fields to search
  $("#btn-find").click(function (e) {
    // prevents a refresh based on a value
    e.preventDefault();
    // clear the tag list before showing the new tags
    document.getElementById("tags").options.length = 0;

    // grab the user typed pattern
    var taggerS = $("#tag-pattern").val();
    if ($("#tag-pattern").val() != "") {
      //console.log("made it inside: " + taggerS);
      // query the database to get a list of tags that match the query.
      IMWS.get("v1/metadata/tags", {
        pattern: taggerS,
        grouppattern: "XMP",
        searchkey: true,
      }).then(
        function (response) {
          // build the drop down multi-select box
          var select = document.getElementById("tags");
          var options = [];
          var option = document.createElement("option");

          for (var i = 0; i < response.tags.length; ++i) {
            option.text = option.value = response.tags[i].key;
            options.push(option.outerHTML);
          }

          select.insertAdjacentHTML("beforeEnd", options.join("\n"));
          //console.log("results: " + JSON.stringify(response.tags, null, 2));
        },
        function (error) {
          //console.log ("we erred: "+JSON.stringify(error, null, 2));
        }
      );
    }
  });
});

///////////////////////////////////////////////////////////////////
// We monitor IMatch events.
function setupMonitoring() {
  if (IMatch.isEmbedded()) {
    IMatch.openWebSocket();
  } else {
    // Running in the browser
    IMatch.openWebSocket(IMatch.DEFAULT_WS_URL);
  }

  // Monitor IMatch events to get notice about file window events.
  IMatch.webSocket().onmessage = function (e) {
    var msg = JSON.parse(e.data);
    // check to see if the scope changed - if it did, update the total number of files in the window
    if (msg.msg == "fileWindow.scopeChanged") {
      IMWS.get("v1/files", {
        idlist: IMatch.idlist.fileWindowFiles,
        fields: "id",
      }).then(function (response) {
        filesWindow = response.files.length;
      });
    }

    if (msg.msg == "app.appfilter.removed") {
      // An app filter was removed. This may be the
      // filter we have set, or another one.
      if (msg.data.appId == APP_ID) {
        // Our filter. Remove the clear filter button.
        $("#btn-clear-filter").hide();
      }
    }
  };
}

///////////////////////////////////////////////////////////////////
// Patternmatching to allow for the AND OR AND NOT advanced searches using +/-/|
function convertString() {
  config.allMatchPattern = $("#all-match-pattern").val();

  //console.log ("match: " + config.allMatchPattern);
  config.allMatchPattern = config.allMatchPattern.replace(/\+/g, "AND ");
  config.allMatchPattern = config.allMatchPattern.replace(/\-/g, "OR ");
  config.allMatchPattern = config.allMatchPattern.replace(/\|/g, "AND NOT ");
  //console.log ("match after: " + config.allMatchPattern);
}

///////////////////////////////////////////////////////////////////
// Load the configuration file from the database

function loadConfig() {
  $("#srctags").empty();
  console.log("tags: " + JSON.stringify(config.tags.length, null, 2));

  if (config.tags.length == 0) {
    IMatch.userInfo().then(function (response) {
      DATA_CONFIG = DATA_CONFIG.replace("{u}", response.userName);
      IMWS.dataGet({ name: DATA_CONFIG }).then(function (response) {
        var data = JSON.parse(response.value);
        config = data;
        $.each(config.tags, function (i, p) {
          $("#srctags").append($("<option></option>").val(p).html(p));
        });
      });
    });
  } else {
    $.each(config.tags, function (i, p) {
      $("#srctags").append($("<option></option>").val(p).html(p));
    });
  }
}

///////////////////////////////////////////////////////////////////
// Save the configuration to the database securing the tag list
function saveConfig() {
  //go through each tag in the tag list and setup a temp array
  $("#srctags > option").each(function () {
    var temp = this.text;
    maintags.push(temp);
  });

  let uniqueChars = [];
  maintags.forEach((c) => {
      if (!uniqueChars.includes(c)) {
          uniqueChars.push(c);
      }
  });


  // write the config back to the database
  config.tags = uniqueChars;
  IMWS.dataSet({
    name: DATA_CONFIG,
    value: JSON.stringify(config),
  });
}

///////////////////////////////////////////////////////////////////
// Open or update a result window to show the results.
function openResultWindow(title) {
  //console.log("Configuration: " + JSON.stringify(config, null, 2));
  if (config.reuseResultWindow && resultWindowInstanceId != 0) {
    // Reuse the existing result window if it still exists (may have been closed by the user).
    // We also update it when nothing was found!
    IMatch.updateResultWindow({
      instanceid: resultWindowInstanceId,
      name: title === undefined ? "" : title,
      id: matches.join(","),
    }).then(
      function (response) {
        // Success
      },
      (error) => {
        // Does not exist anymore. We call this again to open a new window.
        resultWindowInstanceId = 0;
        $("#btn-reuse").attr("disabled", true);
        openResultWindow(title);
      }
    );
  } else {
    //console.log("in here: " + JSON.stringify(matches, null, 2));
    IMatch.openResultWindow({
      name:
        title === undefined
          ? IMatchTranslate.getStr("T_RESULT_WINDOW_NAME")
          : title,
      id: matches.join(","),
    }).then(function (response) {
      resultWindowInstanceId = response.instanceId;
    });
  }
}

///////////////////////////////////////////////////////////////////
// Search for metadata across all tags
//   
function runSearch() {
  config.mode = 4;
  //saveConfig();
  running = true;

  convertString();
  
  if (config.allMatchPattern.length == 0) return;

  // build a comma seperated list of tags from the multi-select drop down.
    var value2 = [];
  $("#srctags > option").each(function () {
    value2.push(this.text);
  });
  var values = value2.join(",");
  var maintags = value2;

  $("#btn-running").show();
  var startTime = performance.now();
  // We either search the entire database, or search the active file window for the pattern defined and only search the provided tags from the drop down multi-select
  // I am restricting the search to the "XMP" group as tags are only shown for this group anyway - this should speed up the search signficantly!
  IMatch.get("v1/search/metadata", {
    scope: $("#search-scope").val() == "db" ? "database" : "idlist",
    idlist: IMatch.idlist.fileWindowFilesTotal,
    pattern: config.allMatchPattern,
    groups: "XMP",
    tags: values,
    advancedmode: true,
    //tags: "hierarchicalkeywords",
    fields: "id",
  }).then(
    function (response) {
        var endTime = performance.now();
        var totaltime = (endTime-startTime)/1000;
      // setup the variables to build the results window header - total files in the search range
      if ($("#search-scope").val() == "db") {
        filesRange = filesTotal;
      } else {
        filesRange = filesWindow;
      }
      
      matches = response.files;
      $("#btn-running").hide();
      $("#timing").val('Search time was ' + totaltime.toFixed(4) + ' seconds long and we found ' + matches.length + ' matching files from searching '+filesRange+' files');
      $("#timing").show();


      complete(
        true,
        "Found: [" +
          matches.length +
          "/" +
          filesRange +
          "] -> Search: " +
          config.allMatchPattern
      );
    },
    function (error) {
      complete(false);
    }
  );
}

///////////////////////////////////////////////////////////////////
// This is called when all files have been processed or
// when the app runs into an error.
function complete(success, title) {
  running = false;

  if (success) {
    if (config.scope == "fw" && config.fwMode == "fwm-filter") {
      // Filter the active file window. We include both the matches and the currently selected files!
      if (matches.length > 0) {
        IMatch.get("v1/files", {
          idlist: IMatch.idlist.fileWindowSelection,
          fields: "id",
        }).then(function (response) {
          // merge the selected files with the matches
          for (let f of response.files) {
            if (matches.indexOf(f) < 0) {
              matches.push(f);
            }
          }
          IMatch.fileWindowFilterUpdate({
            appid: IMatch.appId() != undefined ? IMatch.appId() : APP_ID,
            mode: "include",
            id: matches.join(","),
          });
          $("#btn-clear-filter").fadeIn();
        });
      }
    } else {
      // Show results in a result window
      if (matches.length > 0) {
        openResultWindow(title);
      }
    }
  }
}

